from tokenize import Name
from selenium import webdriver
from selenium.webdriver.edge.service import Service
from selenium.webdriver.edge.options import Options
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.common.action_chains import ActionChains

from jbtutils_selenium.constants import DRIVERS_FOLDER
import pathlib

script_directory = pathlib.Path().absolute()

try:
    if not EDGE: EDGE = None
except NameError:
    pass

def open(log_level='3', profile_dir=None, arguments=[], experimental_options=[]):
    ser = Service(DRIVERS_FOLDER+'\msedgedriver.exe')

    op = Options()
    op.add_argument(f'--log-level={log_level}')
    if profile_dir:
        op.add_argument(f'user-data-dir={script_directory}\\{profile_dir}')

    for arg in arguments:
        op.add_argument(arg)

    for exp_op in experimental_options:
        op.add_experimental_option(exp_op['key'], exp_op['value'])

    global EDGE
    EDGE = webdriver.Edge(service=ser, options=op)
    EDGE.maximize_window()
    return EDGE


def wait(secs=10):
    return WebDriverWait(EDGE, secs)

def navigate_to(url):
    EDGE.get(url)

def back():
    EDGE.back()

def forward():
    EDGE.forward()

def switch_to_frame(iframe):
    return EDGE.switch_to.frame(iframe)

def set_page_load_timeout(secs):
    EDGE.set_page_load_timeout(secs)

def set_page_load_strategy(strategy):
    EDGE.capabilities['pageLoadStrategy'] = strategy

def set_implicit_wait(secs):
    EDGE.implicitly_wait(secs)

def execute_js(js_str, return_val=False):
    if return_val:  return EDGE.execute_script('return '+js_str)
    else:           EDGE.execute_script(js_str)

def execute_js_on_element(element, js_str):
    EDGE.execute_script(js_str, element)



def accept_alert(wait_secs=3):
    wait(secs=wait_secs).until(EC.alert_is_present())
    EDGE.switch_to.alert.accept()

    
def find_element(search_by, val):
    return EDGE.find_element(search_by, val)

def find_elements(search_by, val):
    return EDGE.find_elements(search_by, val)


def get_current_window():
    return EDGE.current_window_handle

def get_windows():
    return EDGE.window_handles

def get_window(index):
    return EDGE.window_handles[index]

def switch_to_window(window):
    return EDGE.switch_to.window(window)

def close_current_window():
    EDGE.close()

def get_current_window_title():
    return EDGE.title

def get_current_url():
    return EDGE.current_url


def get_actions():
    return ActionChains(EDGE)
