import logging
from time import time, sleep

from jbtutils_selenium.drivers.edge import find_elements as find_els, find_element as find_el
from jbtutils_selenium.elements.common import get_search_by


def find_elements(val, by='xpath', wait_secs=10, ignore_error=False, interval=1):
    logging.debug(f'Looking for all elements by {by} = {val}')

    search_by = get_search_by(by)

    initial_time = time()
    while True:
        logging.debug('Trying to find elements')

        elements = find_els(search_by, val)
        if elements:
            logging.debug('Found elements')
            return elements

        if time()-initial_time >= wait_secs:
            if ignore_error:
                logging.debug(f'Timed out waiting for elements but ignoring due to {ignore_error=}')
                return None
            else:
                e = TimeoutError('Timed out waiting for elements')
                logging.exception(e)
                raise e

        sleep(interval)


def find_element(val, by='xpath', wait_secs=10, ignore_error=False, interval=1):
    logging.debug(f'Looking for all element by {by} = {val}')

    search_by = get_search_by(by)

    initial_time = time()
    while True:
        logging.debug('Trying to find element')

        element = find_el(search_by, val)
        if element:
            logging.debug('Found element')
            return element

        if time()-initial_time >= wait_secs:
            if ignore_error:
                logging.debug(f'Timed out waiting for element but ignoring due to {ignore_error=}')
                return None
            else:
                logging.exception('Timed out waiting for element')
                raise TimeoutError('Timed out waiting for element')

        sleep(interval)


def find_elements_by_attribute(search_by, search_val, condition, attribute_name, attribute_val, elements=None, ignore_error=False, wait_secs_any_element=10, wait_secs_these_elements=10, interval=1, just_one=False):
    logging.debug(f'Iterating through elements {search_by} = {search_val} looking for attribute name "{attribute_name}" with value "{attribute_val}"')

    if elements: elements_available = elements
    return_elements = []
    
    initial_time = time()
    while True:
        logging.debug('Searching for elements with the desired attributes')

        if not elements:
            elements_available = find_elements(by=search_by, val=search_val, wait_secs=wait_secs_any_element, interval=interval)

        logging.debug(f'Found {len(elements_available)} elements matching {search_by}={search_val}')
        for element in elements_available:
            logging.debug(f'Checking element: {element}')
            if attribute_name.split(' ')[0] == 'selenium':
                logging.debug('Checking selenium attributes')
                if condition == 'equals':
                    if  (attribute_name == 'selenium accessible name'   and attribute_val == element.accessible_name)               or \
                        (attribute_name == 'selenium text'              and attribute_val == element.text)                          or \
                        (attribute_name == 'selenium aria role'         and attribute_val == element.aria_role):
                        logging.debug(f'Found element where {attribute_name} = {attribute_val}')
                        if element not in return_elements: return_elements.append(element)
                elif condition == 'contains':
                    if  (attribute_name == 'selenium accessible name'   and attribute_val in element.accessible_name)               or \
                        (attribute_name == 'selenium text'              and attribute_val in element.text)                          or \
                        (attribute_name == 'selenium aria role'         and attribute_val in element.aria_role):
                        logging.debug(f'Found element where {attribute_name} contains {attribute_val}')
                        if element not in return_elements: return_elements.append(element)
            else:
                logging.debug('Checking DOM attributes')
                attr_val = element.get_attribute(attribute_name)
                if (condition == 'equals' and attr_val == attribute_val) or (condition == 'contains' and attribute_val in attr_val):
                    if      condition == 'equals':      logging.debug(f'Found element where {attribute_name} = {attribute_val}')
                    elif    condition == 'contains':    logging.debug(f'Found element where {attribute_name} contains {attribute_val}')   
                    if element not in return_elements: return_elements.append(element)
                else:
                    logging.debug(f'No DOM element found with atribute name "{attribute_name}" and value "{attribute_val}"')

            logging.debug(f'So far have found {len(return_elements)} element(s) with these characteristics')

            if just_one and len(return_elements) == 1:
                logging.debug(f'Only one element requested due to {just_one=}. Returning this element')
                return return_elements[0]

        if time()-initial_time >= wait_secs_these_elements:
            logging.debug(f'Times up searching for these specific elements, found {len(return_elements)} elements')
            break

    if return_elements: return return_elements
    if ignore_error:    return None

    e = TimeoutError(f'Could not find any element where elements {search_by} = {search_val} looking for attribute name "{attribute_name}" with value "{attribute_val}"')
    logging.exception(e)
    raise e
