import logging
from time import time, sleep

from selenium.webdriver.support import expected_conditions as EC
from selenium.common.exceptions import TimeoutException

from jbtutils_selenium.drivers.edge import wait, get_current_url, get_windows


def wait_for_new_window(num_windows=2, wait_secs=10, ignore_error=False):
    logging.debug('Waiting for new window')
    try:
        return wait(wait_secs).until(EC.number_of_windows_to_be(num_windows))
    except TimeoutException:
        if ignore_error:    return None
        else:               raise TimeoutException


def wait_for_window_title_contains(text, wait_secs=10, ignore_error=False):
    logging.debug(f'Waiting for window to contain in the title: {text}')
    try:
        return wait(wait_secs).until(EC.title_contains(text))
    except TimeoutException:
        if ignore_error:    return None
        else:               raise TimeoutException

def wait_for_window_url_contains(url, wait_secs=10, ignore_error=False, interval=1):
    logging.debug(f'Waiting for window url to contain: {url}')

    initial_time = time()
    current_url = None
    while True:
        current_url = get_current_url()
        if url in current_url:
            return True
        
        if time()-initial_time >= wait_secs:
            if ignore_error:    return None
            else:               raise TimeoutException

        sleep(interval)

def wait_for_window_to_close(window, wait_secs=10, ignore_error=False, interval=0.5):
    logging.debug(f'Waiting for window to close: {window}')
    
    initial_time = time()
    while True:

        windows = get_windows()
        if not window in windows:
            return True
        
        if time()-initial_time >= wait_secs:
            if ignore_error:    return None
            else:               raise TimeoutException

        sleep(interval)
